@@ -1,10 +1,12 @@
 #' Time-series fluctuations in sound level
 #'
-#' @name noise_fluct
-#' @docType data
-#' @author Junta Tagusari \email{j.tagusari@@eng.hokudai.ac.jp}
-#' @keywords time-series sound noise traffic
-#' @description
 #' Results of the measurement of the sound level,
 #' where peaks due to road traffic are observed.
-NULL
+#' @author Junta Tagusari \email{j.tagusari@@eng.hokudai.ac.jp}
+#' @keywords time-series sound noise traffic
+#' @format A data frame with 32,001 rows and 4 columns:
+#' \describe{
+#'   \item{time}{time}
+#'   \item{f500, f1000, f2000}{Octave-band sound levels whose center frequencies are 500, 1000 and 2000 Hz.}
+#' }
+"noise_fluct"
